@@ -25,7 +25,8 @@
 #define TLSCONTEXT_H_INCLUDED
 
 #include "syslog-ng.h"
-
+#include "messages.h"
+#include "atomic.h"
 #include <openssl/ssl.h>
 
 typedef enum
@@ -52,6 +53,7 @@ typedef enum
   TSO_NOTLSv1=0x0004,
   TSO_NOTLSv11=0x0008,
   TSO_NOTLSv12=0x0010,
+  TSO_NOTLSv13=0x0020,
 } TLSSslOptions;
 
 typedef enum
@@ -68,13 +70,19 @@ typedef struct _TLSContext TLSContext;
 #define X509_MAX_O_LEN 64
 #define X509_MAX_OU_LEN 32
 
+typedef struct _TLSVerifier
+{
+  GAtomicCounter ref_cnt;
+  TLSSessionVerifyFunc verify_func;
+  gpointer verify_data;
+  GDestroyNotify verify_data_destroy;
+} TLSVerifier;
+
 typedef struct _TLSSession
 {
   SSL *ssl;
   TLSContext *ctx;
-  TLSSessionVerifyFunc verify_func;
-  gpointer verify_data;
-  GDestroyNotify verify_data_destroy;
+  TLSVerifier *verifier;
   struct
   {
     int found;
@@ -84,8 +92,18 @@ typedef struct _TLSSession
   } peer_info;
 } TLSSession;
 
-void tls_session_set_verify(TLSSession *self, TLSSessionVerifyFunc verify_func, gpointer verify_data,
-                            GDestroyNotify verify_destroy);
+
+#define TLSCONTEXT_ERROR tls_context_error_quark()
+GQuark tls_context_error_quark(void);
+
+enum TLSContextError
+{
+  TLSCONTEXT_UNSUPPORTED,
+};
+
+#define TMI_ALLOW_COMPRESS 0x1
+
+void tls_session_set_verifier(TLSSession *self, TLSVerifier *verifier);
 void tls_session_free(TLSSession *self);
 
 TLSContextSetupResult tls_context_setup_context(TLSContext *self);
@@ -93,22 +111,37 @@ TLSSession *tls_context_setup_session(TLSContext *self);
 void tls_session_set_trusted_fingerprints(TLSContext *self, GList *fingerprints);
 void tls_session_set_trusted_dn(TLSContext *self, GList *dns);
 
-TLSContext *tls_context_new(TLSMode mode);
-void tls_context_free(TLSContext *s);
+TLSContext *tls_context_new(TLSMode mode, const gchar *config_location);
+TLSContext *tls_context_ref(TLSContext *self);
+void tls_context_unref(TLSContext *self);
+TLSVerifier *tls_verifier_new(TLSSessionVerifyFunc verify_func, gpointer verify_data,
+                              GDestroyNotify verify_data_destroy);
+TLSVerifier *tls_verifier_ref(TLSVerifier *self);
+void tls_verifier_unref(TLSVerifier *self);
 
+
+void tls_session_configure_allow_compress(TLSSession *tls_session, gboolean allow_compress);
 gboolean tls_context_set_verify_mode_by_name(TLSContext *self, const gchar *mode_str);
 gboolean tls_context_set_ssl_options_by_name(TLSContext *self, GList *options);
 gint tls_context_get_verify_mode(const TLSContext *self);
 void tls_context_set_verify_mode(TLSContext *self, gint verify_mode);
 void tls_context_set_key_file(TLSContext *self, const gchar *key_file);
 void tls_context_set_cert_file(TLSContext *self, const gchar *cert_file);
+gboolean tls_context_set_keylog_file(TLSContext *self, gchar *keylog_file_path, GError **error);
 void tls_context_set_pkcs12_file(TLSContext *self, const gchar *pkcs12_file);
 void tls_context_set_ca_dir(TLSContext *self, const gchar *ca_dir);
 void tls_context_set_crl_dir(TLSContext *self, const gchar *crl_dir);
+void tls_context_set_ca_file(TLSContext *self, const gchar *ca_file);
 void tls_context_set_cipher_suite(TLSContext *self, const gchar *cipher_suite);
+gboolean tls_context_set_tls13_cipher_suite(TLSContext *self, const gchar *tls13_cipher_suite, GError **error);
+void tls_context_set_sigalgs(TLSContext *self, const gchar *sigalgs);
+void tls_context_set_client_sigalgs(TLSContext *self, const gchar *sigalgs);
 void tls_context_set_ecdh_curve_list(TLSContext *self, const gchar *ecdh_curve_list);
 void tls_context_set_dhparam_file(TLSContext *self, const gchar *dhparam_file);
+void tls_context_set_sni(TLSContext *self, const gchar *sni);
 const gchar *tls_context_get_key_file(TLSContext *self);
+EVTTAG *tls_context_format_tls_error_tag(TLSContext *self);
+EVTTAG *tls_context_format_location_tag(TLSContext *self);
 
 void tls_log_certificate_validation_progress(int ok, X509_STORE_CTX *ctx);
 gboolean tls_verify_certificate_name(X509 *cert, const gchar *hostname);
